C ***************************** BAINITE4.FOR **********************
C
C ******************* BAINITE TRANSFORMATION KINETICS **********
C
C ------ Version 3, 21 December 2022, H. K. D. H. Bhadeshia ----------
C
Composition of each phase assumed to be homogeneous. 
Carbide precipitation not included.
C Publication open access: https://doi.org/10.1051/jp4:1997506 
C Finish temperature set at 20 degrees centigrade
C
C Calculates the martensite-start temperatures, ensuring that
C  the transformation temperature to bainite is not set below
C  the Ms temperature.
C
C Calculates the Ms temperature of the residual austenite and
C  at the same time, the fraction of that austenite which is
C  expected to decompose to martensite, according to Khan &
C  Bhadeshia, and to Koistinen and Marburger. The austenite 
C  retained is calculated assuming that the sample cools to 
C  20 degrees centigrade
C
C
C The Ms temperature of the residual austenite is presented
C   for each temperature interval during continuous cooling
C   transformation, or for each time interval during isothermal
C   transformation.



C
C Search5 adaption of Gethin's model Search1, this has corrected integration, 
C no longer looks for constants, calculates some values previously required
C as input and has a continuous cooling option. N Chester 7/11/95
C Search6 as version 5 but now takes into account prior ferrite due to 
C intercritical annealing.
C Search7: XTO400 is used rather than XTO, thus the bainite transformation 
C does not start before Bs. 15/12/95
C Search8: temperature inputs all in deg.C and NEWF contains SNDP.
C Search9: linear change in U (proportional platelet size) where U=0.5 at
C 330C and U=1 at 405C.
C Search10: MAXVOL using XTO rather than XTO400.
C speed1: rearranged and new more sensible output style by Harry. 14/7/96
C MAV introduced (MAXVOL when allow for allot. ferrite) and ISO adjusted
C to allow for allot. ferrite. 16/7/96
C Note that in MUCGREAD, temperature is output as Kelvin (KTEMP)
C speed version3. Now complicated continuous cooling added (need file 
C "cooling" with time (from start) and temperature pairs) 26/7/96. 
C speed version4. COOL2 corrected so that zero time at start temperature.
C Can have problems hanging at the end due to a very high Ms temperature.
C It this happens change the AB4 and ICALC numbers to cut the cooling step
C into a different number and it will work. Also constants are not read in
C anymore. N Chester 16/9/96.
C Units of K1'/u corrected to mm-1 s 9/10/96
C size of dimentiones used in COOL2 increased to 1000 so as to prevent 
C errors 10/10/96
C
C
       PROGRAM MAIN
C
       IMPLICIT DOUBLE PRECISION (A-H,K-Z),INTEGER (I,J)
       DIMENSION CNT(4),C(8),K(8),AV(1000),ACTEMP(1000),AU(1000),
     &                  AXTO(1000),AXTO400(1000),COTEMP(1000),
     &                  COTIME(1000),ATIME(1000)
C
C
      WRITE(*,22)
      READ (*,*) IN
22    FORMAT (20X,' ** Heat treatment details for Bainite **'//
     &  22X,' Isothermal transformation                   = 1'/
     &  22X,' Time to isothermally achieve given fraction = 2'/
     &  22X,' Continuous cooling one step                 = 3'/
     &  22X,' Continuous cooling many steps               = 4'/) 

      IF(IN .EQ. 3) CALL AREAD(ST,FT,TIME2)   
      IF(IN .EQ. 3) KTEMP=ST
C      
C number each temperature step is divided into for continuous cooling 
      IAB4=20
C
      IF(IN .EQ. 4) THEN
       CALL RCOOL(IAB4,J2,JS,COTEMP,COTIME)
       KTEMP=COTEMP(1)
C
C so that printout is correct:
       ST=COTEMP(1)
       FT=COTEMP(J2)
       TIME2=COTIME(J2)
      ENDIF
C
       CALL MUCGREAD(C,K,KBAR,DGM,KTEMP,XTO400,XTO,
     &               XBAR,ND,FF,T10,T20,W,W1,IN,MS0)
C
           OPEN (UNIT=3,FILE='out')
           OPEN (UNIT=22, FILE='ruf')
C
C set version number
           AVER=4.0
C
C Set kinetic constants
           CNT(1)=0.34458D+02
           CNT(2)=0.14750D+03
           CNT(3)=0.20980D+05
           CNT(4)=0.30327D+02
C
C----------------------------------------------------------------
          IF(IN .EQ. 1 .OR. IN .EQ. 2)THEN
C
C                Calculate the maximum fraction of bainite during 
C                isothermal transformation at KTEMP,allowing for
C                the carbon enrichment caused by prior ferrite
C
                 MAXVOL=MV (KTEMP,XTO,XBAR)
                 MAV=MAXVOL*FF
C
                 IF (XTO400 .EQ. 0.0000D+00) GOTO 40
                 IF (XBAR .GT. XTO400) GOTO 50
C
                 WRITE(3,3) AVER, KBAR,K(2),K(3),K(4),K(5)
     &                ,K(6),K(7),ND,(1.0-FF),K(1),KTEMP-273,MAV
                 WRITE(*,3) AVER, KBAR,K(2),K(3),K(4),K(5)
     &                ,K(6),K(7),ND,(1.0-FF),K(1),KTEMP-273,MAV
          ELSE
                 WRITE(3,12) AVER, KBAR,K(2),K(3),K(4),K(5)
     &                ,K(6),K(7),ND,(1.0-FF),K(1)
                 WRITE(*,12) AVER, KBAR,K(2),K(3),K(4),K(5)
     &                ,K(6),K(7),ND,(1.0-FF),K(1)
          ENDIF
C----------------------------------------------------------------

          WRITE (3,2) CNT(1),CNT(2), CNT(3),CNT(4)
          WRITE (*,2) CNT(1),CNT(2), CNT(3),CNT(4)
C
             PSI=0.0D0
             CALL ISO (IN,VA,MAV,FF)
C
C
C           --------------------------------------------------
C              Calculate time to achieve specified fraction
C           --------------------------------------------------
              IF (IN .EQ. 2) THEN
20              PSI=VA/MAXVOL
                IF (VA .GE. MAXVOL) GOTO 9
                 TIME=NEWF(KTEMP,DGM,PSI,MAXVOL,ND,CNT,XBAR,U)
                 VA=VA*FF

                 WRITE(*,8) XBAR,XTO,XTO400
                 WRITE(3,8) XBAR,XTO,XTO400
                 WRITE (*,5)
                 WRITE (3,5)
                 WRITE (3,15) VA,TIME,U
                 WRITE (*,15) VA,TIME,U
                 XRICH=(XBAR+XALPH(KTEMP)*(1.0D+00-FF+VA))/(FF-VA)
                     IF(KTEMP .gt. 500.0+273.15)THEN
                       JTLOW=140+273
                       JTHIGH=500+273
                     ELSE
                       JTLOW=KTEMP-80
                       JTHIGH=KTEMP+80
                     ENDIF
                 CALL MSTART(T10,T20,W,W1,JTLOW,JTHIGH,MS,XRICH,0)
                      CALL MART(MS,20.0D+00,FF-VA,VMART,VAUSTR)
                      CALL MART2(MS,20.0D+00,FF-VA,VMART2,VAUSTR2)
                      WRITE(*,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
                      WRITE(3,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
                 GOTO 9
                ENDIF
C           --------------------------------------------------
C              Calculate fraction during continuous cooling
C           --------------------------------------------------
    
             IF (IN .GE. 3) THEN
                CALL COOL2 (K,ND,CNT,FF,AV,ACTEMP,AU,AXTO,AXTO400,
     &           ATIME,ICALC,ST,FT,TIME2,J2,COTIME,COTEMP,IN)
                 WRITE(3,6)ST-273,FT-273,TIME2
                 WRITE(*,6)ST-273,FT-273,TIME2
C
C To write out the number of steps in the cooling curve
                IF (IN .EQ. 4) THEN
                 WRITE(3,19)JS
                 WRITE(*,19)JS
                ENDIF
C
                 WRITE (*,7)
                 WRITE (3,7)
                  DO 21 I=1,ICALC
                   KTEMP2=ACTEMP(I)+273
                   VA=AV(I)
                   XRICH=(XBAR+XALPH(KTEMP2)*(1.0D+00-FF+VA))/(FF-VA)
                     IF(ACTEMP(I) .gt. 500.0)THEN
                       JTLOW=140+273
                       JTHIGH=500+273
                     ELSE
                       JTLOW=KTEMP2-80
                       JTHIGH=KTEMP2+80
                     ENDIF
                      IF(MS .gt. -273.0) THEN
                         CALL MSTART(T10,T20,W,W1,JTLOW,
     &                               JTHIGH,MS,XRICH,0)
                      ENDIF
                   WRITE(3,4) AV(I),ACTEMP(I),ATIME(I),AU(I),
     &                        AXTO(I),AXTO400(I),MS
                   WRITE(*,4) AV(I),ACTEMP(I),ATIME(I),AU(I),
     &                        AXTO(I),AXTO400(I),MS
                     IF(MS .LT. FT-273 .AND. I .EQ. ICALC) THEN
                      WRITE(*,14) FT-273,(FF-VA)
                      WRITE(3,14) FT-273,(FF-VA)
                      GOTO 21
                     ENDIF
                     IF(MS .GT. ACTEMP(I)) THEN
                      CALL MART(MS,FT-273,FF-VA,VMART,VAUSTR)
                      CALL MART2(MS,FT-273,FF-VA,VMART2,VAUSTR2)
                      WRITE(*,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
                      WRITE(3,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
                      GOTO 9
                     ENDIF
21                CONTINUE
                  GOTO 9
             ENDIF
C
C           ---------------------------------------------------
C           Calculate fraction during isothermal transformation
C           ---------------------------------------------------
                 WRITE(*,8) XBAR,XTO,XTO400
                 WRITE(3,8) XBAR,XTO,XTO400
                 WRITE (*,16)
                 WRITE (3,16)
             DO 10 I=1,20
                VA=I*0.050D0
                PSI=VA/MAXVOL
                  IF (VA .GE. MAXVOL)THEN
                    VA=VA-0.05
                    GOTO 18
                  ENDIF
                TIME=NEWF(KTEMP,DGM,PSI,MAXVOL,ND,CNT,XBAR,U)
                VA=VA*FF
                 XRICH=(XBAR+XALPH(KTEMP)*(1.0D+00-FF+VA))/(FF-VA)
                     IF(KTEMP .gt. 500.0+273.15)THEN
                       JTLOW=140+273
                       JTHIGH=500+273
                     ELSE
                       JTLOW=KTEMP-80
                       JTHIGH=KTEMP+80
                     ENDIF
                 CALL MSTART(T10,T20,W,W1,JTLOW,JTHIGH,MS,XRICH,0)
                WRITE (3,17) VA,TIME,U,MS
                WRITE (*,17) VA,TIME,U,MS
10           CONTINUE
18                    CALL MART(MS,20.0D+00,FF-VA,VMART,VAUSTR)
                      CALL MART2(MS,20.0D+00,FF-VA,VMART2,VAUSTR2)
                      WRITE(*,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
                      WRITE(3,11)MS,VMART,VMART2,VAUSTR,VAUSTR2
C           --------------------------------------------------
C           --------------------------------------------------
C
       GOTO 9
40     WRITE (*,*) '          ** Temperature above Bs ***'
       GOTO 9
50     WRITE (*,*) 'Enriched carbon content higher than XTO400'
       GOTO 9
C
C
C
C           --------------------------------------------------
C                   Format statements only
C           --------------------------------------------------

1      FORMAT(10X,' Maximum volume fraction of bainite =',
     &         F8.2//)
2      FORMAT(10X, ' Values of model constants '//
     &    15X, 'K1/u (mm-1 s)  = ',D10.2/
     &    15X, 'Lambda 1       = ',D10.2/
     &    15X, 'K2 (J/mol)     = ',D10.2/
     &    15X, 'Lambda 2       = ',D10.2/)
3      FORMAT(///
     &10X,'**********************************************',
     &'**************'//
     &20X,'**   Bainite Transformation Kinetics  **'//
     &20X,'          University of Cambridge '/
     &20X,'    Naomi Chester and Harry Bhadeshia'///
     &10X,' Program Version ',F3.1 /
     &10X,' CHEMICAL COMPOSITION'///
     &10X,' Carbon     ',F6.3,'  wt.%',9X,' Silicon     ',F6.3,'  wt.%'/
     &10X,' Manganese  ',F6.3,'  wt.%',9X,' Nickel      ',F6.3,'  wt.%'/
     &10X,' Molybdenum ',F6.3,'  wt.%',9X,' Chromium    ',F6.3,'  wt.%'/
     &10X,' Vanadium   ',F6.3,'  wt.%'//
     &10X,' Austenite grain size (micrometres)        = ', F8.0/
     &10X,' Allotriomorphic ferrite fraction          = ', F8.2/
     &10X,' Enriched carbon concentration (wt.%)      = ', F8.3/
     &10X,' Isothermal transformation temperature (C) = ', F8.0/
     &10X,' Maximum volume fraction of bainite        = ', F8.2/
     &//)
4     FORMAT (5X,F10.2,F10.1,F12.1,F9.2,2F8.3,F8.0)
5     FORMAT( 20X,'Fraction   Seconds     u     ')
6     FORMAT(10X,' Continuous cooling transformation '/
     &       15X,' Start temperature  (C) = ',F8.0/
     &       15X,' Finish temperature (C) = ',F8.0/
     &       15X,' Cooling time (s)       = ',F8.0/)
7     FORMAT( 10X,'Fraction    CTEMP       Time     u    '
     &            ,' XTO    XTO400   Ms (C)')
8     FORMAT(15X,' XBAR   (mole fraction)   = ',F8.4/
     &       15X,' XTO    (mole fraction)   = ',F8.4/
     &       15X,' XTO400 (mole fraction)   = ',F8.4//)
11    FORMAT(//10X,' Ms of Residual Austenite (C)   = ',F8.0/
     &         10X,' Fraction of martensite         = ',F8.2
     &             , '   to',F7.2/
     &         10X,' Fraction of retained austenite = ',F8.2 
     &             , '   to',F7.2/)

12    FORMAT(///
     &10X,'**********************************************',
     &'**************'//
     &20X,'**   Bainite Transformation Kinetics  **'//
     &20X,'          Cambridge University '/
     &20X,'    Naomi Chester and Harry Bhadeshia'///
     &10X,' Program Version ',F3.1 /
     &10X,' CHEMICAL COMPOSITION'///
     &10X,' Carbon     ',F6.3,'  wt.%',9X,' Silicon     ',F6.3,'  wt.%'/
     &10X,' Manganese  ',F6.3,'  wt.%',9X,' Nickel      ',F6.3,'  wt.%'/
     &10X,' Molybdenum ',F6.3,'  wt.%',9X,' Chromium    ',F6.3,'  wt.%'/
     &10X,' Vanadium   ',F6.3,'  wt.%'//
     &10X,' Austenite grain size (micrometres)            = ', F8.0/
     &10X,' Allotriomorphic ferrite fraction          = ', F8.2/
     &10X,' Enriched carbon concentration (wt.%)      = ', F8.3/
     &//)
14    FORMAT(//10X,' Retained austenite fraction at ',F6.0,' C = '
     &          ,F8.2/
     &       10X,  ' Martensite fraction = 0.00')

15    FORMAT (15X,F10.2,F12.1,F9.2,2F8.3,F8.0)
16    FORMAT( 20X,'Bainite fraction   Seconds     u    Ms (C) ')
17    FORMAT (15X,F17.2,F12.1,F9.2,F8.0)
19    FORMAT (15X,' Number of cooling steps = ',I6//) 
C           --------------------------------------------------
C           --------------------------------------------------

C
C
9       STOP
        END
C
C************************************************************************
C************************************************************************
C      H. K. D. H. Bhadeshia
C 
C AFEG AFEG LN ACTIVITY OF IRON IN GAMMA 
      DOUBLE PRECISION FUNCTION AFEG(XEQ,AJ)
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1)))**6)
      AFEG=TEQ
      RETURN
      END
C
C*****************************************************************
C
C Linear regression
C H. K. D. H. Bhadeshia, 1989
C
C CONST is the best fit slope of the regression line
C CORR is the correlation coefficient
C J  is set to the number of x,y pairs input into the subroutine
C J9 is the number of x,y pairs which are not to be
C   included in the analysis. Only the first J-J9 pairs are analysed
C JP is set to 1 if the regression results need to be printed
C SLOPE is the best fit slope of the regression line
C X is an array of x values
C Y is an array of y values
C
      SUBROUTINE ANALY(J,J9,CONST,SLOPE,CORR,X,Y,JP)
C
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      DOUBLE PRECISION X(100),Y(100)
C
      J8=J-J9
      AX=0.0D+00
      AY=0.0D+00
      AX2=0.0D+00
      AY2=0.0D+00
      AXY=0.0D+00
C
       DO 1 I=1,J8
          AX=AX+X(I)
          AY=AY+Y(I)
          AXY=AXY+X(I)*Y(I)
          AX2=AX2+X(I)*X(I)
          AY2=AY2+Y(I)*Y(I)
1      CONTINUE
C
      CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
      SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
      CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*
     &(J8*AY2-AY*AY)))
C
      IF(JP .EQ. 1)WRITE(*,2)CONST,SLOPE,CORR,JP
2     FORMAT('   INTERCEPT=',D12.4,'    SLOPE=',D10.3,
     &'    CORRELATION=',F8.4, I10)
C
        RETURN
        END

C*******************************************************************
C Subroutine for heat treatment parameters during continuous cooling
C
      SUBROUTINE AREAD(ST,FT,TIME)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
C
C read start and finish temperatures, cooling time
9         WRITE (*,2)
           CALL REED (ST)
           FT=20.00D+00
C          WRITE (*,3)
C           CALL REED (FT)
            IF (ST .LT. FT) THEN
              WRITE (*,4) 
              CALL REED (FT)
            ENDIF
          WRITE (*,5)
            CALL REED(TIME)
C
C convert to absolute temperatures
      ST=ST+273.0D0
      FT=FT+273.0D0
C
2     FORMAT(30X,' Start temperature  (C) ?')
C3     FORMAT(30X,' Finish temperature (C) ?')
4     FORMAT(30X,' Start temperature must exceed finish temperature'/
     &       35X,' Enter finish temperature again ')
5     FORMAT(30X,' Cooling time (s) ?')
C
      RETURN
      END

C***********************************************************************
C     H. K. D. H. Bhadeshia
      SUBROUTINE AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,STORE,J)
      IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
      J=0
1     DFTO=FTO1(H,S,XTO,T,W,W1,H1,S1,F,AJ,AJ1)+STORE
      J=J+1
      IF (DABS(DFTO) .LE. 10.0D+00) GOTO 2
      G9=G91(XTO,T,W,W1,H1,S1,F,H,S,AJ,AJ1)
      IF (J .GE. 9) GOTO 2
      XTO=XTO-DFTO/G9
      IF (XTO .LE. 1.0D-12) GOTO 3
      GOTO 1
3     XTO=0.0000D+00
2     RETURN
      END
C
C*******************************************************************
      SUBROUTINE BOUND(A,B,C)
      DOUBLE PRECISION A,B,C
2      WRITE(*,1)B,C
1     FORMAT(12X,' Input value out of bounds'/
     &14X,' The limits are ',D12.4,' to ', D12.4/)
      CALL REED(A)
      IF(A .LT. B .OR. A .GT. C)GOTO 2
      RETURN
      END
C

C*****************************************************************
C      H. K. D. H. Bhadeshia
C
C CG CG DOUBLE PRECISION FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
      DOUBLE PRECISION FUNCTION CG(X,T,W,R)
      DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
      J=1.0-DEXP(-W/(R*T))
         IF(X .LE. 1.0D-10) THEN
           CG=DLOG(1.0D-10)
         ELSE
           DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
           DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0)-(
     &     13.48)*T)/(R*T)
           CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
         ENDIF
      RETURN
      END
C**************************************************************
C Continuous cooling calculations.
C
      SUBROUTINE COOL2(K,ND,CNT,FF,AV,ACTEMP,AU,
     &AXTO,AXTO400,ATIME,
     &ICALC,ST,FT,TIME,J2,COTIME,COTEMP,IN)

      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION CNT(4),C(8),K(8),AV(1000),ACTEMP(1000),
     &AU(1000),AXTO(1000),AXTO400(1000),COTEMP(1000),
     &COTIME(1000),ATIME(1000)
C
C   ICALC divides the temperature interval into discrete steps (if IN=3)
C     ICALC cannot be increased beyond 1000 without modifying the
C     matrix dimensions in both subroutine and main program
         ICALC=21
C
C   Reset matrices
C
        DO 1 I=1,1000
            AV(I)=0.0D+00
            ACTEMP(I)=0.0D+00
            AU(I)=0.0D+00
            AXTO(I)=0.0D+00
            AXTO400(I)=0.0D+00
1        CONTINUE

C
      VA=0.0D0
      PSI=0.0D0
C
      IF (IN .EQ. 4) THEN
         ICALC=J2
         GOTO 333
      ENDIF
C
C
      DTIME=0.0
      DTEMP=ST-FT
      DTEMP=DTEMP/(ICALC-1)
C
C---------------------------------------------------------
333   DO 10 J=1,ICALC
          X=J-1
      IF (IN .EQ. 4) THEN
          TEMP=COTEMP(J)
          IF(J .GT. 1) DTIME=COTIME(J)-COTIME(J-1)
      ELSE
          TEMP=ST-(X*DTEMP)
          IF(J .GT. 1) DTIME=TIME/(ICALC-1)
      ENDIF
          CALL MUCG(C,K,TEMP,ND,DGM,XTO400,XTO,XBAR,T10,T20,W,W1)
             IF (XTO400 .EQ. 0.0000D+00) THEN
               V=0.0000D+00
               CTEMP=TEMP-273.0D0
               GOTO 20
             ENDIF
          MAXVOL=MV (TEMP,XTO,XBAR)
           WRITE (22,*) MAXVOL, TEMP, XBAR, I
          PSI=VA/MAXVOL
          PTIME=NEWF(TEMP,DGM,PSI,MAXVOL,ND,CNT,XBAR,U)
          TTIME=PTIME+DTIME
          CALL ROUGH(TEMP,DGM,MAXVOL,ND,CNT,XBAR,TTIME,VOL)
          CTEMP=TEMP-273.0D0
          V=VOL*FF
          VA=VOL
C
20          AV(J)=V
            ACTEMP(J)=CTEMP
            AU(J)=U
            AXTO(J)=XTO
            AXTO400(J)=XTO400
C
            IF (IN .EQ. 3) THEN
              ATIME(J)=DTIME*(J-1)
            ELSE
              ATIME(J)=COTIME(J)
            ENDIF
C
10    CONTINUE
C
C---------------------------------------------------------
C
      RETURN
      END
C
C*****************************************************************
C     H. K. D. H. Bhadeshia
C     ENERGY ENERGY
      DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
      DOUBLE PRECISION T,T10,T20,F,T7
      T7=T-100*T20
      IF (T7 .LT. 300) GOTO 1
      IF (T7 .LT. 700) GOTO 2
      IF (T7 .LT. 940) GOTO 3
      F=-8.88909+0.26557*(T7-1140)-1.04923D-3*((T7-1140)**2)
      F=F+2.70013D-6*((T7-1140)**3)-3.58434D-9*((T7-1140)**4)
      GOTO 4
1     F=1.38*T7-1499
      GOTO 4
2     F=1.65786*T7-1581
      GOTO 4
3     F=1.30089*T7-1331
4     ENERGY=(141.0*T10 + F)*4.187
      RETURN
      END
C
C******************************************************************
C    Calculates the factorial of number passed to it
C
      DOUBLE PRECISION FUNCTION FACT(IY)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z)
      IMPLICIT INTEGER (I-K)
      A=1.0D0
      DO 20 J=1,IY
      A=J*A
20    CONTINUE
      FACT=A
      END
C

C******************************************************************
C Finer search for volume fraction for a given temperature.
C
      SUBROUTINE FINE(TEMP,DGM,MAXVOL,ND,CNT,XBAR,TTIME,VHIGH,VOL)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION CNT(4)
C
       VLOW=VHIGH-0.01D0
         DO 30 I=1,100
           VA=VLOW+(I*0.00010D0)
20         PSI=VA/MAXVOL
           TIME=NEWF(TEMP,DGM,PSI,MAXVOL,ND,CNT,XBAR,U)
           IF (TIME .GT. TTIME) GOTO 31
30       CONTINUE
C
31     VOL=VA
C
       RETURN
       END
C

C*****************************************************************
C
      DOUBLE PRECISION FUNCTION FTO1
     &(H,S,X,T,W,W1,H1,S1,F,AJ,AJ1)
      DOUBLE PRECISION R,X,T,T60,ZEN1,ZEN2,ZENER,W,W1,H1,S1,F,
     &AJ,AJ1,D,D11,H,S
      D=DSQRT(1-2*(1+2*AJ)*X+(1+8*AJ)*X*X)
      D11=DSQRT(9-6*X*(2*AJ1+3)+(9+16*AJ1)*X*X)
      T60=T*(1-X)/(28080*X)
      IF (T60 .GT. 1.0) GOTO 18
      IF (T60 .GT. 0.25) GOTO 17
      ZEN1=0.2307+42.7974*T60-233.8631*T60*T60+645.4485*T60*T60*T60
     &-954.3995*(T60**4)+711.8095*(T60**5)-211.5136*(T60**6)
      ZEN2=-2.6702+45.6337*T60-225.3965*T60*T60+567.7112*(T60**3)
     &-771.6466*(T60**4)+538.1778*(T60**5)-151.3818*(T60**6)
      GOTO 16
17    ZEN2=1
      ZEN1=3.295
16    ZENER=(((ZEN2*X)**2)*(-50898.56)/(1-X))+ZEN1*T*X*0.6623741
      ZENER=ZENER*4.187
      GOTO 15
18    ZENER=0.0
15    R=8.31432
      FTO1=X*R*T*DLOG(X*X)+X*(H1-(H)-(S1-(S)
     &)*T+4*W1-6*W)-R*T*(1-X)*DLOG((1-X)**4)+5*R*T*(1-2*X)*DLOG(1-2*X)
     &-R*T*X*DLOG(((D-1+3*X)/(D+1-3*X))**6)-R*T*(1-X)*DLOG(((1-2*AJ+(4
     &*AJ-1)*X-D)/(2*AJ*(2*X-1)))**6)+3*R*T*X*DLOG(3-4*X)+R*T*
     &X*DLOG(((D11-3+5*X)/(D11+3-5*X))**4)+(1-X)*F+ZENER
      RETURN
      END
C
C
C************************************************************
C   Function which is an infinite series (20 steps done here)
C
      DOUBLE PRECISION FUNCTION FUNCT(X)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z)
      IMPLICIT INTEGER (I-K)
C
      B=0.0D0
        DO 10 I=1,20
          A=FACT(I)
          C=X**I
          B=(C/(A*I))+B
10      CONTINUE
      FUNCT=B
C
      END
C
C

C**********************************************************************
C     H. K. D. H. Bhadeshia
C
      SUBROUTINE GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      IMPLICIT REAL*8(A-H,K-Z)
C CALCULATION OF THE OPTIMUM NUCLEUS C CONTENT AND ACTIVITY OF C IN
C FERRITE NUCLEUS
C
C Estimate a value for X
C February 1991
       X=6.3998D-07*T-3.027D-04
       IF(X .LT. 0.1D-07)X=1.0D-08
       IF(X .GT. 0.32D-03)X=0.32D-03
C Estimation complete
C
      AJ1=1.0D+00-DEXP(-W1/(R*T))
1     D1=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     &+(9.0D+00+16.0D+00*AJ1)*X*X)
      B1=((D1-3.0D+00+5.0D+00*X)/(D1+3.0D+00-5.0D+00*X))
      B2=((3.0D+00-4.0D+00*X)/X)
      B3=(H1 - S1*T + 4.0D+00*W1)/(R*T)

      A1=DLOG(B1*B1*B1*B1*B2*B2*B2)+B3
C
      IF(X .GT. 1.0D-08)THEN
        A1FE=DLOG(1.0D+00-X)
      ELSE
        A1FE=X
      ENDIF
C
      TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
C
C Newton iteration
C
      IF (DABS(TEST) .GT. 10.0) THEN
        DA1=(3.0D+00*X/(3.0D+00-4.0D+00*X))*(
     &  (4.0D+00*X-3.0D+00)/(X*X)-4.0D+00/X)
        DA2=(0.5D+00/D1)*(-12.0D+00*AJ1-18.0D+00+
     &  18.0D+00*X+32.0D+00*AJ1*X)
        DA2=4.0D+00*(((DA2+5.0D+00)/(D1-3.0D+00+5.0D+00*X))
     &  -((DA2-5.0D+00)/(D1+3.0D+00-5.0D+00*X)))
        DA1=DA1+DA2
        DA1FE=1.0D+00/(X-1.0D+00)
        ERROR=TEST/(R*T*(DA1FE-DA1))
        IF (ERROR .GT. X) ERROR = 0.3D+00*X
        X=DABS(X-ERROR)
        GOTO 1
      ENDIF
C End of iteration
      RETURN
      END
C*****************************************************************
C
C     H. K. D. H. Bhadeshia
      DOUBLE PRECISION FUNCTION G91(XTO,T,W,W1,H1,S1,F,
     &H,S,AJ,AJ1)
      DOUBLE PRECISION R,XTO,T,W,W1,H1,S1,F,AJ,AJ1,FD,FD1,DT5,
     &DT6,DZEN1,DZEN2,DZEN3,V1,V2,V3,V4,V5,V6,V7,V8,V9,G1,
     &DZEN6,DZEN7,DZEN8,H,S
      R=8.31432
      FD=DSQRT(1-2*(1+2*AJ)*XTO+(1+8*AJ)*XTO*XTO)
      FD1=DSQRT(9-6*XTO*(2*AJ1+3)+(9+16*AJ1)*XTO*XTO)
      DT5=28080*XTO/(1-XTO)
      DT6=T/DT5
      IF (DT6 .GT. 1.0) GOTO 48
      IF (DT6 .LT. 0.25) GOTO 47
      DZEN1=0.2307+42.7974*DT6-233.8631*(DT6**2)+645.4485*(DT6**3)
     &-954.3995*(DT6**4)+711.8095*(DT6**5)-211.5136*(DT6**6)
      DZEN2=-2.6702+45.6337*DT6-225.3965*(DT6**2)+567.7112*(DT6**3)
     &-771.6466*(DT6**4)+538.1778*(DT6**5)-151.3818*(DT6**6)
      GOTO 46
47    DZEN2=1
      DZEN1=3.295
46    DZEN3=(((DZEN2*XTO)**2)*(-50898.56)/(1-XTO))+DZEN1*T*XTO*
     &(0.6623741)
      DZEN3=DZEN3*4.187
      GOTO 45
48    DZEN3=0.0
45    V1=FD-1+3*XTO
      V2=FD+1-3*XTO
      V3=1-2*AJ+(4*AJ-1)*XTO-FD
      V4=2*AJ*(2*XTO-1)
      V5=FD1-3+5*XTO
      V6=FD1+3-5*XTO
      V7=(XTO-1-2*AJ+8*XTO*AJ)/FD
      V8=(9*XTO-9-6*AJ1+16*AJ1*XTO)/FD1
      V9=H1-(H)-(S1-(S))*T-6*W+4*W1
      G1=2+DLOG(XTO**2)+4+DLOG((1-XTO)**4)-10-DLOG((1-2*XTO)**10)
     &-DLOG((V1/V2)**6)-6*((XTO/V1)*(V7+3+V1*(3-V7)/V2))
     &+DLOG((V3/V4)**6)-6*((1-XTO)/V3)*(4*AJ*(1-(V3/V4))-V7)
     &+3*(DLOG(3-4*XTO)-4*XTO/(3-4*XTO))+DLOG((V5/V6)**4)+4*(XTO
     &/V5)*(V8+5+(V5/V6)*(5-V8))
      IF (DT6 .GT. 1.0) GOTO 90
      DZEN6=(-3.3948+13.6112*DT6-13.4376*(DT6**2))*T/(28080*
     &((1-XTO)**2))
      DZEN7=(-3.3118+15.7462*DT6-23.2449*(DT6**2))*T/(28080*
     &((1-XTO)**2))
      DZEN8=50898*((DZEN2*XTO)**2)/((1-XTO)**2)+(-50898*(2*
     &DZEN2*DZEN6*(XTO**2)+2*XTO*(DZEN2**2))/(1-XTO))+DZEN1
     &*T*0.6623741+DZEN7*T*XTO*0.6623741
      GOTO 91
90    DZEN8=0.0
91    G91=V9+R*T*G1+DZEN8-F
      RETURN
      END
C
***********************************************************************
C Intercritical annealing calculations.
C
       SUBROUTINE INTAN (FGAMMA,C,CGAMMA)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z),INTEGER (I,J)
       DIMENSION C(8)
C 
C FGAMMA is the fraction of austenite in intercritically annealed sample
C VPRIOR is the ferrite at intercritical annealing temperature
C CGAMMA is the carbon concentration of residual austenite (wt %)
C CALPHA is the solubility of carbon in ferrite at 
C   intercritical temperature (wt %)
C
        CBAR=C(1)
        CGAMMA=CBAR
C
          WRITE (*,1)
          READ (*,*) ANS
          IF(ANS .EQ. 0) THEN
            FGAMMA=1.0D+00
            GOTO 4
          ENDIF
C
          WRITE (*,2)
           CALL REED(VPRIOR)
           IF(VPRIOR .LT. 0.0D-00 .OR. VPRIOR .GT. 1.0D+00)
     &                   CALL BOUND(VPRIOR,0.0D-00,1.0D+00)

          WRITE (*,3)
            READ (*,*) CTEMP
            KTEMP=CTEMP+273.15
C
C Calculate C concentration in ferrite, crudely convert to wt%
C
              CALPHA=(1201.15/55.84)*XALPH(KTEMP)
C
C Estimate carbon-enrichment of austenite due to prior ferrite
C
              FGAMMA = 1.0D+00 - VPRIOR
              CGAMMA = (CBAR-CALPHA*VPRIOR)/(FGAMMA)
C
1      FORMAT(30X,' Intercritically annealed ? (Yes = 1, No = 0)')
2      FORMAT(30X,' Fraction of allotriomorphic ferrite ?')
3      FORMAT(30X,' Intercritical anneal temperature (C) ?')

4     RETURN
      END
C********************************************************************
C Reads in which mode is required. 
C
      SUBROUTINE ISO(IN,VOL,MAV,FF)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
C
C      WRITE(*,2)
C      READ (*,*) IN
      IF (IN .EQ. 1) GOTO 10
      IF (IN .EQ. 2) GOTO 20
      IF (IN .EQ. 3) GOTO 30
C
10    VOL=0.0D0
      RETURN
C
20     WRITE (*,1)
       CALL REED (VF)
        IF (VF .LT. 0.0D0 .OR. VF .GT. MAV)
     &    CALL BOUND(VF,0.0D0,MAV)
        VOL=VF*1.0D0
        VOL=VOL/FF
      RETURN
C
30    VOL=0.0D0

C
1     FORMAT (30X,' Volume fraction of bainite ?')
C2     FORMAT (20X,' ** Heat treatment details for Bainite **'//
C     &  22X,' Isothermal transformation                   = 1'/
C     &  22X,' Time to isothermally achieve given fraction = 2'/
C     &  22X,' Continuous cooling transformation           = 3'/)

      RETURN
      END
C
C **************************************************************
C H. K. D. H. Bhadeshia
C 
C To calculate the fraction of martensite as a function of the 
C  undercooling below the Ms temperature using the method of
C  S. A. Khan and H. K. D. H. Bhadeshia
C  Materials Science and Engineering A A129 (1990) 257-272
C
      SUBROUTINE MART(MS,TQ,VAUST,VMART,VAUSTR)
C
      IMPLICIT DOUBLE PRECISION (A-H,K-Z)
C
C guess a value for the volume fraction of martensite, VMART
        IF(MS-TQ .GT. 70.0D+00)THEN
          VMART=0.5D+00
        ELSE
          VMART=0.05D+00
        ENDIF
C
1     A=MARTA(VMART,MS,TQ)
      B=MARTB(VMART)
        IF(DABS(A) .GT. 0.01D+00)THEN
          VMART=DABS(VMART - A/B)
          IF(VMART .GE. 1.0D+00)VMART=0.99D+00  
         GOTO 1
        ENDIF
C
      VMART=VMART*VAUST
      VAUSTR=VAUST-VMART
C
      RETURN
      END
C ***************************************************************
C H. K. D. H. Bhadeshia
C
C  Function subroutine for estimating MARTA  
C
C 
C To help calculate the fraction of martensite as a function of the 
C  undercooling below the Ms temperature using the method of
C  S. A. Khan and H. K. D. H. Bhadeshia
C  Materials Science and Engineering A A129 (1990) 257-272
C
      DOUBLE PRECISION FUNCTION MARTA(VMART,MS,TQ)
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
C
      M=0.0029D+00
      MARTA=(-(DLOG(1.0D+0-VMART))/VMART)-M*(MS-TQ)-1.0D+0
C
      RETURN
      END
C ***************************************************************
C H. K. D. H. Bhadeshia
C
C 
C To help calculate the fraction of martensite as a function of the 
C  undercooling below the Ms temperature using the method of
C  S. A. Khan and H. K. D. H. Bhadeshia
C  Materials Science and Engineering A A129 (1990) 257-272
C
C  Function subroutine for estimating GDASH (derivative of MARTA)
C
      DOUBLE PRECISION FUNCTION MARTB(VMART)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z)
C
      MARTB=(1.0D+0/((1.0D+0-VMART)*VMART))
     &+((DLOG(1.0D+0-VMART))/(VMART*VMART))
C
      RETURN
      END
C **************************************************************
C H. K. D. H. Bhadeshia
C
C To  calculate the fraction of martensite as a function of the 
C  undercooling below the Ms temperature using the method of
C  D. P. Koistinen and R. E. Marburger
C  Acta Metallurgica 7 (1959) 59
C
      SUBROUTINE MART2(MS,TQ,VAUST,VMART,VAUSTR)
C
C    
      IMPLICIT DOUBLE PRECISION (A-H,K-Z)
C
C guess a value for the volume fraction of martensite, VMART
        VMART=1.0D+00-DEXP(-0.011*(MS-TQ))
        VMART=VMART*VAUST
        VAUSTR=VAUST-VMART
C
      RETURN
      END


C*******************************************************************************
C To calculate the martensite-start temperature
C H. K. D. H. Bhadeshia, University of Cambridge
C
C MS is the martensite-start temperatures in
C   degrees centigrade
C
C JP defines the amount of printing in subroutines called by this
C   subroutine. A value of 1 means no printing.
C JTLOW, JTHIGH  are set
C   They define the range over which calculations are done
C   IT defines the temperature interval at which calculations done
C
C
C Particular caution needed in passing JHIGH & JLOW to MSTART,
C  bearing in mind that these are integers. *****************
C

C T10 and T20 are defined by subroutine OMEGA. They are functions
C   of the substitutional solute content of the alloy.
C W and W1 represent the carbon-carbon interaction energies in
C  austenite and ferrite respectively, in Joules per mole.
C X1 is the carbon concentration in mole fraction
C 
C
C
      SUBROUTINE MSTART(T10,T20,W,W1,JTLOW,JTHIGH,MS,X1,JP)
C
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION DCTEMP(100), FTO(100)
C
C Initialize J1 and J2
C
      J1=0
      J2=0
C
C Guess a value of XTO, the carbon concentration (mole fraction) given by the
C   T-zero curve of the Temperature versus Carbon plot (phase diagram)
C
      XTO=0.07D+00
C
C R is the universal gas constant
C
      R=8.3143D+00
C
C H is the excess partial molar enthalpy of solution of carbon in austenite
C S is the excess partial molar entropy of solution of carbon in austenite
C H is in Joules per mole, S is in Joules per mole per K
C G. J. Shiflet, J. R. Bradley and H. I. Aaronson, Metallurgical Transactions
C   A, 9A (1978) 999.
C
        H=38575.0D+00
        S=13.48D+00
C
C This is to ensure that JTLOW, JTHIGH, JT are not changed on exit
C
       ITLOW=JTLOW
       ITHIGH=JTHIGH
2       IT=(JTHIGH-JTLOW)/20
        IF(IT .LT. 5)THEN
           ITHIGH=5*IT+ITLOW
           GOTO 2
        ENDIF

        IF(ITLOW .LT. 413)THEN
          ITLOW=413
          ITHIGH=513
        ENDIF

C
C
C H1 is the excess partial molar enthalpy of solution of carbon in ferrite
C S1 is the excess partial molar entropy of solution of carbon in ferrite
C H1 is in Joules per mole, S1 is in Joules per mole per K
C J. A. Lobo and  G. H. Geiger, Metallurgical Transactions A 7A (1976),
C  pages 1347 and 1359.
C
      DO 1 J= ITLOW,ITHIGH,IT
C
          J1=J1+1
          KTEMP=J
C
          IF (KTEMP .GE. 1000.0) THEN
              H1=105525.0D+00
              S1=45.34521D+00
          ELSE
              H1=111918.0D+00
              S1=51.44D+00
          ENDIF
C
C Calculate the free energy change for the transformation of austenite
C  to ferrite of the same chemical composition
C
          F=ENERGY(KTEMP,T10,T20)
          AJ=1.0D+00-DEXP(-W/(R*KTEMP))
          AJ1=1.0D+00-DEXP(-W1/(R*KTEMP))
          FTO(J1)=FTO1(H,S,X1,KTEMP,W,W1,H1,S1,F,AJ,AJ1)
C
C Calculate the temperature in degrees centigrade
C
          DCTEMP(J1)=KTEMP-273.0D+00
C
1     CONTINUE
C
C Linear regression to obtain the free energy change for the transformation
C  of austenite to ferrite of the same chemical composition, as a
C  function of the temperature
C
      CALL ANALY(J1,0,CONST,SLOPE,CORR,DCTEMP,FTO,JP)
C
C
C Calculate the martensite-start temperatures
C Modification made February 1991 to allow for large carbon concentrations
C Based on Metal Science paper on thermodynamics of martensitic transformations
C in plain carbon steels
C
C H. K. D. H. Bhadeshia, Metal Science 15 (1981) 175-177
C H. K. D. H. Bhadeshia, Metal Science 15 (1981) 178-180
C
      XMS=X1
      IF(XMS .GT. 0.0594D+00)XMS=0.0594D+00
      MS=(-1086.9D+00 + 2941.8D+00*XMS - 9.6016D+05*XMS*XMS
     &+ 2.5382D+07*XMS*XMS*XMS - 1.8488D+08*XMS*XMS*XMS*XMS
     & - CONST)/SLOPE
      IF(MS .le. -273.00)MS=-273.15
C
C
C
C
        RETURN
        END


C************************************************************
C This subroutine calculates GMAX and XTO400 (volume fraction of bainite at 
C T0' line). Adaptation of MUCG46, N Chester 19/10/95
C
      SUBROUTINE MUCG(C,K,T,ND,GMAX,XTO400,XTO,XBAR,T10,T20,W,W1)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8),K(8)
C
      CALL OMEGA(C,K,W,XBAR,T10,T20,0)
      FTO=-1.0
      X1=C(1)
      XA=0.001D+00 
      R=8.31432D+00
      W1=48570.0D+00
      H=38575.0D+00
      S=13.48D+00
      XEQ=0.2D+00
      XTO=0.07D+00
      XTO400=0.06D+00
C
      CTEMP=T-273.00D+00
      IF (T .LE. 1000.0) GOTO 22
      H1=105525.0D+00
      S1=45.34521D+00
      GOTO 19
22    H1=111918.0D+00
      S1=51.44D+00
19    F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      AJ1=1.0D+00-DEXP(-W1/(R*T))
      A=CG(X1,T,W,R)
      AFE=AFEG(X1,AJ)
C
      CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
      GMAX=R*T*(A1-A)
      CALL AXTO(H,S,H1,S1,XTO400,T,W,W1,F,AJ,AJ1,400.0D+00,J99)
      CALL AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,0.0D+00,J98)
C
      RETURN
      END

C
C************************************************************************
C This subroutine reads in composition, temperature and grain size.
C
      SUBROUTINE MUCGREAD(C,K,KBAR,GMAX,T,XTO400,XTO,
     &                    XBAR,ND,FF,T10,T20,W,W1,IN,MS0)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION C(8),K(8)
C
C
      WRITE(*,707)
707   FORMAT(19X,' Carbon wt.% ?')
      CALL REED(K(1))
      IF(K(1) .LT. 0.1D-03 .OR. K(1) .GT. 2.0D+00)
     &CALL BOUND(K(1),0.1D-03,2.0D+00)
       KBAR=K(1)
      WRITE(*,708)
      CALL REED(K(2))
      IF(K(2) .LT. 0.0D+00 .OR. K(2) .GT. 2.5D+00)
     &CALL BOUND(K(2),0.0D+00,2.5D+00)
708   FORMAT(19X,' Silicon wt.% ?')
      WRITE(*,709)
      CALL REED(K(3))
      IF(K(3) .LT. 0.0D+00 .OR. K(3) .GT. 3.5D+00)
     &CALL BOUND(K(3),0.0D+00,3.5D+00)
709   FORMAT(19X,' Manganese wt.% ?')
      WRITE(*,710)
      CALL REED(K(4))
      IF(K(4) .LT. 0.0D+00 .OR. K(4) .GT. 4.5D+00)
     &CALL BOUND(K(4),0.0D+00,4.5D+00)
710   FORMAT(19X,' Nickel wt.% ?')
711   FORMAT(19X,' Molybdenum wt.% ?')
      WRITE(*,711)
      CALL REED(K(5))
      IF(K(5) .LT. 0.0D+00 .OR. K(5) .GT. 1.5D+00)
     &CALL BOUND(K(5),0.0D+00,1.5D+00)
712   FORMAT(19X,' Chromium wt.% ?')
      WRITE(*,712)
      CALL REED(K(6))
      IF(K(6) .LT. 0.0D+00 .OR. K(6) .GT. 3.5D+00)
     &CALL BOUND(K(6),0.0D+00,3.5D+00)
713   FORMAT(19X,' Vanadium wt.% ?')
      WRITE(*,713)
      CALL REED(K(7))
      IF(K(7) .LT. 0.0D+00 .OR. K(7) .GT. 1.5D+00)
     &CALL BOUND(K(7),0.0D+00,1.5D+00)
715   FORMAT(19X,' Grain Size micrometres ?')
      WRITE(*,715)
      CALL REED(ND)
C
      IF(IN .GE. 3) THEN
        CALL INTAN (FF,K,CRICH)
        K(1)=CRICH
C       not happy with conversion below. Better method? ***
        XRICH=(55.84/1201.15)*CRICH
        CALL MUCG (C,K,T,ND,GMAX,XTO400,XTO,XBAR,T10,T20,W,W1)
        JTLOW=140+273
        JTHIGH=T
        CALL MSTART(T10,T20,W,W1,JTLOW,JTHIGH,MS0,XRICH,1)
        IF(MS0+273 .gt. T) THEN
         WRITE(*,2)MS0
         CALL REED(CTEMP1)
         T=CTEMP1+273
        ENDIF
        RETURN
      ENDIF

714   FORMAT(23X,' Isothermal transformation temperature deg.C ?'/)
      WRITE(*,714)
      CALL REED(TEMP)

      TEMP=TEMP+273
      T=TEMP*1.0D+00
      CALL INTAN (FF,K,CRICH)
      K(1)=CRICH
C       not happy with conversion below. Better method? ***
        XRICH=(55.84/1201.15)*CRICH
      CALL MUCG (C,K,T,ND,GMAX,XTO400,XTO,XBAR,T10,T20,W,W1)
        JTLOW=140+273
        JTHIGH=T
        CALL MSTART(T10,T20,W,W1,JTLOW,JTHIGH,MS0,XRICH,1)
        IF(MS0+273 .gt. T) THEN
         WRITE(*,3)MS0
         CALL REED(CTEMP1)
         T=CTEMP1+273
        ENDIF
C
2     FORMAT(10X,' The start temperature is below Ms  (',F8.0,')'/
     &       10X,' Please input new value of start Temperature (C)'/)
3     FORMAT(10X,' Isothermal trans. temperature below Ms (',F8.0,')'/
     &    10X,' Input new value of transformation temperature (C)'/)
C
      RETURN
      END
C

C************************************************************************
C Calculates maximum volume fraction of ferrite at To'line.
C
       DOUBLE PRECISION FUNCTION MV (T,XTO,XBAR)
       IMPLICIT DOUBLE PRECISION (A-H,L-Z),INTEGER (I-K)
         XA=XALPH(T)
         MV=(XTO-XBAR)/(XTO-XA)
       END
C
C

C***************************************************************************
C
C
C
C  NEWF the t=function corrected from Gethin's paper
C
      DOUBLE PRECISION FUNCTION NEWF (TK,GM0,PSI,THETA,ND,CD,XB,U)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z),INTEGER (I-K)
      PARAMETER (R=8.314D0,RR=2540D0)
      DIMENSION CD(4)
      C1=CD(1)
C
C   Linear change in U, half at 330 deg.C than at 405 deg.C.
C
      U=(TK-528)/150

C Modified 13-7-96, Harry, 
C to ensure a lower limit to plate size
       IF(U .lt. 0.3)U=0.3
C
        C1=C1/U
        BETA=CD(2)*(1.0D0- CD(4)*XB)
        C2=CD(3)
        GSM=ND
        GNT= 3.636*(TK-273.15) -2540D0
C
         A=1.0D0/(THETA*GSM*C1)
         B=THETA*BETA
         GAMMA=C2*(GM0-GNT)/(RR*R*TK)
         C=C2/(R*TK)+C2*GM0/(RR*R*TK)
         E=GAMMA/B
         F=A*(B+1.0D0)
         FST=DEXP(C)/F
         GA=1.0D0+(B*PSI)
C
             IF (GA .LT. 0.0D0) THEN
               GA=-GA
             ENDIF
C
          LGA=DLOG(GA)
          GB=(GAMMA*PSI)-E
          FGB=FUNCT(GB)
          GC=-E
          FGC=FUNCT(GC)
          G=LGA+FGB-FGC
          HA=1.0D0-PSI
          LHA=DLOG(HA)
          HB=HA*GAMMA
          FHB=FUNCT(HB)
          FHC=FUNCT(GAMMA)
          H=LHA+FHB-FHC
          SND=G*DEXP(E)-H*DEXP(-GAMMA)

      NEWF=FST*SND

      END
C
C
C************************************************************************
C
C     H. K. D. H. Bhadeshia
      SUBROUTINE OMEGA(C,K,W,XBAR,T10,T20,J)
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
      DOUBLE PRECISION C(8),K(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XBAR
      INTEGER U,J
      B3=0.0D+00
      IF(J .EQ. 1) GOTO 2
      C(8)=K(1)+K(2)+K(3)+K(4)+K(5)+K(6)+K(7)
      C(8)=100.0D+00-C(8)
      C(8)=C(8)/55.84D+00
      C(1)=K(1)/12.0115D+00
      C(2)=K(2)/28.09D+00
      C(3)=K(3)/54.94D+00
      C(4)=K(4)/58.71D+00
      C(5)=K(5)/95.94D+00
      C(6)=K(6)/52.0D+00
      C(7)=K(7)/50.94D+00
2     B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
      DO 107 U=2,7
      Y(U)=C(U)/C(8)
107   CONTINUE
      DO 106 U=1,8
      C(U)=C(U)/B1
106   CONTINUE
      XBAR=C(1)
      B2=0.0D+00
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
      T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
      P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     &+3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
      P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     &2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
      P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     &-2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
      P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     &-1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
      P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     &+8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
      P(7)=2011.9996-6247.9118*C(7)+5411.7566*C(7)**2
     &+250118.1085*C(7)**3-4.1676D+06*C(7)**4
      DO 108 U=2,7
      B3=B3+P(U)*Y(U)
      B2=B2+Y(U)
108   CONTINUE
      IF (B2 .EQ. 0.0D+00) GOTO 455
      W=(B3/B2)*4.187
      GOTO 456
455   W=8054.0
456   CONTINUE
      RETURN
      END
C
C********************************************************************
C To read in continuous cooling many steps option. The time temperature
C pairs are read from file "cooling".The time in seconds and the 
C temperature in Deg.C with a limit of 100 lines.
C Typical "cooling" file:
C	0	600
C	125	350
C	175	300
C	300	20
C
C
      SUBROUTINE RCOOL(IAB4,J2,J3,COTEMP,COTIME)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION COTEMP(1000),COTIME(1000),RTEMP(100),RTIME(100)
C
      OPEN (UNIT=4, FILE='cooling')
C
      DO 47 J=1,100
         READ(4,*,END=51) RTIME(J), RTEMP(J)
47     CONTINUE
51    J3=J-1
C
C To divide each time and temperature interval into AB4 steps (this is 
C set in the main program). STEMP and STIME are arrays of the step sizes.
C COTIME and COTEMP are arrays of the time temp pairs for the calculation. 
C
      DO 48 JA=1,J3-1
       STEMP=(RTEMP(JA)-RTEMP(JA+1))/IAB4
       STIME=(RTIME(JA+1)-RTIME(JA))/IAB4
C
        DO 49 IA=1,IAB4
         I=AB4*(JA-1)+IA
         COTEMP(I)=RTEMP(JA)-STEMP*(IA-1)
         COTIME(I)=RTIME(JA)+STIME*(IA-1)
49      CONTINUE
48    CONTINUE
C
      J2=AB4*(J3-1)+1
         COTEMP(J2)=RTEMP(J3)
         COTIME(J2)=RTIME(J3)
C
C To convert the temperature into K.
C
       DO 52 J5=1,J2
          COTEMP(J5)=COTEMP(J5)+273.0D0
52     CONTINUE
C
       RETURN
       END
C
C********************************************************************
C     H. K. D. H. Bhadeshia
      SUBROUTINE REED(A)
      DOUBLE PRECISION A
996   READ(*,*,ERR=999)A
          GOTO 998
999       WRITE(*,997)
997       FORMAT(19X,' Incorrect Input. Try again'/)
          GOTO 996
998   RETURN
      END
C***********************************************************************
C     H. K. D. H. Bhadeshia
      SUBROUTINE REEDI(I)
      INTEGER I
996   READ(*,*,ERR=999)I
          GOTO 998
999       WRITE(*,997)
997       FORMAT(19X,' Incorrect Input. Try again'/)
          GOTO 996
998   RETURN
      END
C********************************************************************
C Roughly searches for volume fraction for a given temperature.
C
      SUBROUTINE ROUGH(TEMP,DGM,MAXVOL,ND,CNT,XBAR,TTIME,VOL)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)
      DOUBLE PRECISION CNT(4)
C
       DO 10 I=1,100
       VA=I*0.010D0
20     PSI=VA/MAXVOL
       IF (VA .GE. MAXVOL) GOTO 9
       TIME=NEWF(TEMP,DGM,PSI,MAXVOL,ND,CNT,XBAR,U)
       IF (TIME .GT. TTIME) GOTO 11
10     CONTINUE
9      WRITE (*,200) 'MAXIMUM VOLUME FRACTION REACHED'
       WRITE (22,200) 'MAXIMUM VOLUME FRACTION REACHED'
       GOTO 12
11     VHIGH=VA
       CALL FINE(TEMP,DGM,MAXVOL,ND,CNT,XBAR,TTIME,VHIGH,VOL)
200    FORMAT (A33)

12     RETURN
       END
C

C***********************************************************************
C Function giving the equilibrium mole fraction of carbon in ferrite
C based on Harry's paper on first order quasichemical theory, Metal Science
C     H. K. D. H. Bhadeshia
C
      DOUBLE PRECISION FUNCTION XALPH(T)
      IMPLICIT DOUBLE PRECISION (A-H,L-Z),INTEGER (I-K)
C
        CTEMP=(T-273.0D+00)/900.0D+00
C
      XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP*CTEMP
     &-0.2417D-01*CTEMP*CTEMP*CTEMP+
     &0.6966D-02*CTEMP*CTEMP*CTEMP*CTEMP
C
      RETURN
      END
C
